FUNCTION BRIGHT_M, W, R

;+
; DESCRIPTION:
;    Compute brightness temperature given monochromatic Planck radiance.
;
; USAGE:
;    RESULT = BRIGHT_M(W, R)
;
; INPUT PARAMETERS:
;    W           Wavelength (microns)
;    R           Planck radiance (Watts per square meter per steradian
;                per micron)
;
; OUTPUT PARAMETERS:
;    BRIGHT_M    Brightness temperature (Kelvin)
;-

;- Fundamental Physical Constants from http://physics.nist.gov/cuu/Constants/

;- Planck constant (Joule second)
h = 6.62606896d-34

;- Speed of light in vacuum (meters per second)
c = 299792458.0d

;- Boltzmann constant (Joules per Kelvin)      
k = 1.3806504d-23

;- Derived constants      
c1 = 2.0d * h * c * c
c2 = (h * c) / k

;- Convert wavelength to meters
ws = (1.0d-6 * w) > 1.0d-10
      
;- Compute brightness temperature
temp = c2 / (ws * alog(c1 / (1.0d+6 * r * ws^5) + 1.0d))
return, temp > 1.0d-10

END
