FUNCTION BRITE_M, V, R

;+
; DESCRIPTION:
;    Compute brightness temperature given monochromatic Planck radiance.
;
; USAGE:
;    RESULT = BRITE_M(V, R)
;
; INPUT PARAMETERS:
;    V         Wavenumber (inverse centimeters)
;    R         Planck radiance (milliWatts per square meter per
;              steradian per inverse centimeter)
;
; OUTPUT PARAMETERS:
;    BRIGHT_M    Brightness temperature (Kelvin)
;-

;- Fundamental Physical Constants from http://physics.nist.gov/cuu/Constants/

;- Planck constant (Joule second)
h = 6.62606896d-34

;- Speed of light in vacuum (meters per second)
c = 299792458.0d

;- Boltzmann constant (Joules per Kelvin)      
k = 1.3806504d-23

;- Derived constants      
c1 = 2.0d * h * c * c
c2 = (h * c) / k

;- Convert wavenumber to inverse meters
vs = (1.0d+2 * v) > 1.0d-10
      
;- Compute brightness temperature
temp = c2 * vs / alog(c1 * vs^3 / (1.0d-5 * r) + 1.0d)
return, temp > 1.0d-10

END
