;---------------------------------------------------------------------
PRO PLANCK_CALC_EVENT, EVENT

print, '(planck_calc_event)'

;- Get state information
widget_getinfo, event, info
mode = info.mode
temp_fmt = info.temp_fmt
rad_fmt = info.rad_fmt

;- Get current values of wavelength/wavenumber, temperature, and radiance
widget_getfloat, info.field_wave, 1.0e-10, 1.0e+10, temp_fmt, wave
widget_getfloat, info.field_temp, 1.0e-10, 1.0e+10, temp_fmt, temp
widget_getfloat, info.field_rad,  1.0e-10, 1.0e+10, rad_fmt, rad

;- Get name of widget which caused the event
widget_control, event.id, get_uvalue=uvalue

;- Units event
if (uvalue eq 'Units') then begin

  ;- Check if units were changed
  if (event.index ne mode) then begin

    ;- Get new units (0=wavelength, 1=wavenumber)
    mode = event.index
    info.mode = mode

    ;- Convert wavelength/wavenumber to inverse units
    wave = 10000.0 / wave
    widget_setfloat, info.field_wave, 1.0e-10, 1.0e+10, temp_fmt, wave

    ;- Set units label
    label = (mode eq 0) ? 'W/m2/sr/um' : 'mW/m2/sr/cm-1'
    widget_control, info.label_units, set_value=label

    ;- Compute new radiance
    rad = (mode eq 0) ? planck_m(wave, temp) : planc_m(wave, temp)
    widget_setfloat, info.field_rad, 1.0e-10, 1.0e+10, rad_fmt, rad

  endif
  
endif

;- Wavelength/wavennumber or temperature event
if (uvalue eq 'Wavelength' or uvalue eq 'Temperature') then begin

  ;- Compute new radiance
  rad = (mode eq 0) ? planck_m(wave, temp) : planc_m(wave, temp)
  widget_setfloat, info.field_rad, 1.0e-10, 1.0e+10, rad_fmt, rad
  
endif

;- Radiance event
if (uvalue eq 'Radiance') then begin

  ;- Compute new temperature
  temp = (mode eq 0) ? bright_m(wave, rad) : brite_m(wave, rad)
  widget_setfloat, info.field_temp, 1.0e-10, 1.0e+10, temp_fmt, temp
  
endif

;- Set state information
widget_setinfo, event, info

END
;---------------------------------------------------------------------
PRO WIDGET_GETINFO, EVENT, INFO

print, '(widget_getinfo)'

widget_control, event.top, get_uvalue=info_ptr
info = *info_ptr

END
;---------------------------------------------------------------------
PRO WIDGET_SETINFO, EVENT, INFO

print, '(widget_setinfo)'

widget_control, event.top, get_uvalue=info_ptr
*info_ptr = info

END
;---------------------------------------------------------------------
PRO WIDGET_GETFLOAT, ID, VALUE_MIN, VALUE_MAX, FORMAT, VALUE

print, '(widget_getfloat)'

widget_control, id, get_value=value
value = float(value[0])
value = (value > value_min) < value_max
widget_control, id, $
  set_value=strcompress(string(value, format=format), /remove_all)

END
;---------------------------------------------------------------------
PRO WIDGET_SETFLOAT, ID, VALUE_MIN, VALUE_MAX, FORMAT, VALUE

print, '(widget_setfloat)'

value = (value > value_min) < value_max
widget_control, id, $
  set_value=strcompress(string(value, format=format), /remove_all)

END
;---------------------------------------------------------------------
PRO WIDGET_CLEANUP, ID

print, '(widget_cleanup)'

widget_control, id, get_uvalue=info_ptr
ptr_free, info_ptr

END
;---------------------------------------------------------------------
PRO PLANCK_CALC

; Interactive Planck function calculator
; Liam.Gumley@ssec.wisc.edu
; 12 June 2007

;- Set startup parameters
mode = 0
wave = 11.0
temp = 300.0
rad = planck_m(wave, temp)
temp_fmt = '(f10.2)'
rad_fmt = '(e14.6)'

;- Top level base
tlb = widget_base(col=1, title='Planck Calculator', $
  xoffset=200, yoffset=200, tlb_frame_attr=1)

;- Top panel
top = widget_base(tlb, row=1)
field_wave = widget_text(top, /editable, xsize=10, uvalue='Wavelength')
widget_setfloat, field_wave, 1.0e-10, 1.0e+10, temp_fmt, wave
list = widget_droplist(top, value=['microns', 'inverse cm'], uvalue='Units')

;- Bottom panel
bot = widget_base(tlb, row=1)

;- Temperature field
base = widget_base(bot, column=1, /frame)
label = widget_label(base, value='Temperature')
field_temp = widget_text(base, /editable, xsize=10, uvalue='Temperature')
widget_setfloat, field_temp, 1.0e-10, 1.0e+10, temp_fmt, temp
label = widget_label(base, value='Kelvin')

;- Radiance field
base = widget_base(bot, column=1, /frame)
label = widget_label(base, value='Radiance')
field_rad = widget_text(base, /editable, xsize=14, uvalue='Radiance')
widget_setfloat, field_rad, 1.0e-10, 1.0e+10, rad_fmt, rad
label_units = widget_label(base, value='W/m2/sr/um     ')

;- Realize widgets
widget_control, tlb, /realize

;- Create info structure
info = {$
  mode:mode, $
  field_wave:field_wave, $
  field_temp:field_temp, $
  field_rad:field_rad, $
  label_units:label_units, $
  temp_fmt:temp_fmt, $
  rad_fmt:rad_fmt}

;- Store info structure
info_ptr = ptr_new(info)
widget_control, tlb, set_uvalue=info_ptr

;- Start event manager
xmanager, 'planck_calc', tlb, /no_block, $
  event_handler='planck_calc_event', cleanup='widget_cleanup'

END
