PRO PLANCK_PLOT, NPTS, WMIN, WMAX, TEMP, MODE, NORM, INIT=INIT

;- Create a Planck function plot

;- Create wavelength/wavenumber data (evenly spaced in log coordinates)
xmin = alog10(wmin)
xmax = alog10(wmax)
w = (dindgen(npts) / double(npts - 1)) * (xmax - xmin) + xmin
w = 10.0d ^ w

;- Create radiance data
if (mode eq 0) then begin
  r = planck_m(w, temp)
endif else begin
  r = planc_m(w, temp)
endelse

;- Normalize if required
if (norm eq 1) then begin
;  if (mode eq 0) then begin
;    r = (w * 1.0d-6) * r
;  endif else begin
;    r = (w * 1.0d+2) * r
;  endelse
  r = r / max(r)
endif

;- Set axis titles
if (mode eq 0) then begin
  xtitle = 'Wavelength (um)'
  ytitle = (norm eq 0) ? 'Radiance (W/m!U2!N/sr/um)' : 'Normalized radiance (no units)'
endif else begin
  xtitle = 'Wavenumber (cm!U-1!N)'
  ytitle = (norm eq 0) ? 'Radiance (mW/m!U2!N/sr/cm!U-1!N)' : 'Normalized radiance (no units)'
endelse

;- Initialize new plot
if keyword_set(init) then begin

  if (norm eq 0) then begin

    ;- Unnormalized plot
    plot, w, r, /ynozero, /xlog, /ylog, background=7, color=0, $
      xrange=[wmin, wmax], xstyle=5, ystyle=4, min_value=1.0d-3, $
      xmargin=[14, 6]
    xtickv = loglevels(10.0d ^ !x.crange)
    axis, /xlog, xaxis=0, xtitle=xtitle, color=0, /save, $
      xgridstyle=1, xticklen=1.0, xticks=n_elements(xtickv) - 1, xtickv=xtickv
    ytickv = loglevels(10.0d ^ !y.crange)     
    axis, /ylog, yaxis=0, ytitle=ytitle, color=0, /save, $
      ygridstyle=1, yticklen=1.0, yticks=n_elements(ytickv) - 1, ytickv=ytickv, $
      ytickformat='(e8.1)'
    
  endif else begin

    ;- Normalized plot
    plot, w, r, /ynozero, /xlog, background=7, color=0, $
      xrange=[wmin, wmax], xstyle=5, ystyle=4, min_value=1.0d-3, $
      xmargin=[12, 6]
    xtickv = loglevels(10.0d ^ !x.crange)
    axis, /xlog, xaxis=0, xtitle=xtitle, color=0, /save, $
      xgridstyle=1, xticklen=1.0, xticks=n_elements(xtickv) - 1, xtickv=xtickv
    axis, yaxis=0, ytitle=ytitle, color=0, /save, $
      ygridstyle=1, yticklen=1.0

  endelse

endif

;- Add plot to existing display
oplot, w, r, color=0

;- Plot temperature label (at maximum radiance location)
rmax = max(r, maxloc)
wmax = w[maxloc]
text = strcompress(string(round(temp), format='(i8)'), /remove_all)
xyouts, wmax, rmax, text, /data, color=0

END
