;---------------------------------------------------------------------
PRO WIDGET_GETINFO, EVENT, INFO

print, '(widget_getinfo)'

widget_control, event.top, get_uvalue=info_ptr
info = *info_ptr

END
;---------------------------------------------------------------------
PRO WIDGET_SETINFO, EVENT, INFO

print, '(widget_setinfo)'

widget_control, event.top, get_uvalue=info_ptr
*info_ptr = info

END
;---------------------------------------------------------------------
PRO WIDGET_CLEANUP, ID

print, '(widget_cleanup)'

widget_control, id, get_uvalue=info_ptr
info = *info_ptr
ptr_free, info.plot_ptr
ptr_free, info_ptr

END
;---------------------------------------------------------------------
PRO WIDGET_GETFLOAT, ID, VALUE_MIN, VALUE_MAX, VALUE

print, '(widget_getfloat)'

widget_control, id, get_value=value
value = float(value[0])
value = (value > value_min) < value_max
widget_control, id, $
  set_value=strcompress(string(value, format='(f10.2)'), /remove_all)

END
;---------------------------------------------------------------------
PRO WIDGET_SETFLOAT, ID, VALUE_MIN, VALUE_MAX, VALUE

print, '(widget_setfloat)'

value = (value > value_min) < value_max
widget_control, id, $
  set_value=strcompress(string(value, format='(f10.2)'), /remove_all)

END
;---------------------------------------------------------------------
PRO PLANCK_TOOL_TLB, EVENT

print, '(planck_tool_tlb)'

;- Get state information
widget_getinfo, event, info

;- Get change in size of top level base
widget_control, event.id, tlb_get_size=base_size
xchange = base_size[0] - info.base_size[0]
ychange = base_size[1] - info.base_size[1]

;- Set new size of draw window
info.xsize = info.xsize + xchange
info.ysize = info.ysize + ychange
widget_control, info.draw_id, xsize=info.xsize, ysize=info.ysize

;- Store new top level base size and display image
widget_control, event.top, tlb_get_size=base_size
info.base_size = base_size

;- Refresh display
planck_tool_display, info, /init

;- Set state information
widget_setinfo, event, info

END
;---------------------------------------------------------------------
PRO PLANCK_TOOL_DISPLAY, INFO, INIT=INIT

print, '(planck_tool_display)'

;- Select draw window
wset, info.draw_window

;- Get plot data
plot_data = *info.plot_ptr

;- Plot data
init = keyword_set(init)
for i = 0, n_elements(plot_data) - 1 do begin
  npts = info.npts
  wmin = plot_data[i].wmin
  wmax = plot_data[i].wmax
  temp = plot_data[i].temp
  mode = info.mode
  norm = info.norm
  planck_plot, npts, wmin, wmax, temp, mode, norm, init=init
  init = 0
endfor

END
;---------------------------------------------------------------------
PRO PLANCK_TOOL_BUTTON, EVENT

print, '(planck_tool_button)'

;- Get state information
widget_getinfo, event, info

;- Get button name
widget_control, event.id, get_value=button

;- Set new mode (Wavelength or Wavenumber)
case button of
  'Wavelength'   : info.mode = 0
  'Wavenumber'   : info.mode = 1
  'Unnormalized' : info.norm = 0
  'Normalized'   : info.norm = 1
endcase

;- Refresh display
planck_tool_display, info, /init

;- Set state information
widget_setinfo, event, info

END
;---------------------------------------------------------------------
PRO PLANCK_TOOL_PLOT, EVENT

print, '(planck_tool_plot)'

;- Get state information
widget_getinfo, event, info

;- Get button name
widget_control, event.id, get_value=value

;- Handle buttons
case value of

  'New Plot' : begin
    widget_getfloat, info.text_wmin, 1.0e-5, 1.0e+7, wmin
    widget_getfloat, info.text_wmax, 1.0e-5, 1.0e+7, wmax
    widget_getfloat, info.text_temp, 1.0e-5, 1.0e+7, temp
    set_plot_data, info.plot_ptr, wmin, wmax, temp, /init
    planck_tool_display, info, /init
  end
  
  'Add Plot' : begin
    widget_getfloat, info.text_wmin, 1.0e-5, 1.0e+7, wmin
    widget_getfloat, info.text_wmax, 1.0e-5, 1.0e+7, wmax
    widget_getfloat, info.text_temp, 1.0e-5, 1.0e+7, temp
    set_plot_data, info.plot_ptr, wmin, wmax, temp
    planck_tool_display, info
  end

  'Save JPEG'  : begin
    filename = dialog_pickfile(group=event.top, /write)
    if (filename ne '') then begin
      widget_control, event.top, hourglass=1
      wset, info.draw_window
      saveimage, filename, /quiet, /jpeg
      widget_control, event.top, hourglass=0
    endif
  end

endcase

;- Set state information
widget_setinfo, event, info
  
END
;---------------------------------------------------------------------
PRO SET_PLOT_DATA, PLOT_PTR, WMIN, WMAX, TEMP, INIT=INIT

;- Store plot data in array of structures

plot_data = [{wmin:wmin, wmax:wmax, temp:temp}]
if keyword_set(init) then begin
  *plot_ptr = plot_data
endif else begin
  *plot_ptr = [*plot_ptr, plot_data]
endelse
help, *plot_ptr

END
;---------------------------------------------------------------------
PRO PLANCK_TOOL, XSIZE=XSIZE, YSIZE=YSIZE, TITLE=TITLE, DEBUG=DEBUG

;- Execute startup code
startup

;- Load graphics colors
loadcolors

;- Check arguments
if (n_elements(xsize) eq 0) then xsize = 800
if (n_elements(ysize) eq 0) then ysize = 600
if (n_elements(title) eq 0) then title = 'Planck Tool'
if (n_elements(debug) eq 0) then debug = 0

;- Initial values
npts = 100
wmin = 0.1
wmax = 50.0
temp = 6000.0
mode = 0
norm = 0

;- Create widgets
tlb = widget_base(row=1, title=title, /tlb_size_events, xoffset=200, yoffset=200)

menu_base = widget_base(tlb, column=1, /frame)

butt_base = widget_base(menu_base, column=1, /exclusive, /frame)
butt = widget_button(butt_base, value='Wavelength', event_pro='planck_tool_button')
if (mode eq 0) then widget_control, butt, /set_button
butt = widget_button(butt_base, value='Wavenumber', event_pro='planck_tool_button')
if (mode eq 1) then widget_control, butt, /set_button

butt_base = widget_base(menu_base, column=1, /exclusive, /frame)
butt = widget_button(butt_base, value='Unnormalized', event_pro='planck_tool_button')
if (norm eq 0) then widget_control, butt, /set_button
butt = widget_button(butt_base, value='Normalized', event_pro='planck_tool_button')
if (norm eq 1) then widget_control, butt, /set_button

text_base = widget_base(menu_base, column=1, /frame)
label = widget_label(text_base, value='Wave Min')
text_wmin = widget_text(text_base, /editable, xsize=10, $
  value=strcompress(string(wmin, format='(f10.2)'), /remove_all))
label = widget_label(text_base, value='Wave Max')
text_wmax = widget_text(text_base, /editable, xsize=10, $
  value=strcompress(string(wmax, format='(f10.2)'), /remove_all))
label = widget_label(text_base, value='Temp (K)')
text_temp = widget_text(text_base, /editable, xsize=10, $
  value=strcompress(string(temp, format='(f10.2)'), /remove_all))

butt = widget_button(menu_base, value='New Plot', event_pro='planck_tool_plot')
butt = widget_button(menu_base, value='Add Plot', event_pro='planck_tool_plot')
butt = widget_button(menu_base, value='Save JPEG', event_pro='planck_tool_plot')

draw_id = widget_draw(tlb, xsize=xsize, ysize=ysize)
widget_control, tlb, /realize

;- Get window id, device name, and top level base size
widget_control, draw_id, get_value=draw_window
wset, draw_window
device = !d.name
widget_control, tlb, tlb_get_size=base_size

;- Store plot data
plot_ptr = ptr_new(0)
set_plot_data, plot_ptr, wmin, wmax, temp, /init

;- Create state information structure
info = {$
  xsize:xsize, $
  ysize:ysize, $
  debug:debug, $
  npts:npts, $
  mode:mode, $
  norm:norm, $
  tlb:tlb, $
  text_wmin:text_wmin, $
  text_wmax:text_wmax, $
  text_temp:text_temp, $
  draw_id:draw_id, $
  draw_window:draw_window, $
  device:device, $
  base_size:base_size, $
  plot_ptr:plot_ptr}

;- Store state information
info_ptr = ptr_new(info)
widget_control, tlb, set_uvalue=info_ptr

;- Update display
planck_tool_display, info, /init

;- Start event manager
xmanager, 'planck_tool', tlb, $
  event_handler='planck_tool_tlb', cleanup='widget_cleanup', /no_block
print, '(planck_tool startup done)'

END
